/************************************************************************************/
/*                                                                                  */
/*     NAME: missdescription.mac.sas                                                */
/*    TITLE: Deskription der fehlenden Werte                                        */
/*     KEYS: descriptive statistics, missing values, frequency tables               */
/*  VERSION: SAS 8.2, gendert auf SAS 9.1                                          */
/*  PRODUCT: BASE, STAT, GRAPH                                                      */
/*    PROCS: FREQ, UNIVARIATE                                                       */
/*                                                                                  */
/*                                                                                  */
/*  Verzeichnis/Dateiname:          F:\missing\missdescription.mac.sas              */
/*  Autoren:                        R. Muche, C. Ring, C. Ziegler                   */
/*  Programmierer:                  Christoph Ziegler                               */
/*  Datum der Ersterstellung:       9/2003                                          */
/*  Aktuelle Makro-Version:         12/2003 gendert auf 08/2005                    */
/*                                                                                  */
/*  Kurzbeschreibung:               Makro zur Deskription der fehlenden Werte       */
/*                                                                                  */
/*  Verwendete externe Untermakros: numberobs.mac.sas                               */
/*                                  exist.mac.sas                                   */
/*                                  exist_var.mac.sas                               */
/*                                  numchar.mac.sas                                 */
/*                                                                                  */
/*  Bentigte Dateien:              Datensatz (Parameter 'data=')                   */
/*                                                                                  */
/*  Erzeugte Dateien:               'path'/univar - 'path'/univarx                  */
/*                                  (falls Parameter 'graphics=1' / 'xvar' gesetzt) */
/*  (Dateinamen nicht frei whlbar) (x ist die Anzahl aller bergebener             */
/*                                  stetigen Variablen, die das Kriterium wie       */
/*                                  unter 'xvar_cat' angegeben erfllen)            */
/*                                                                                  */
/*  Programmnderungen                                                              */
/*  und -ergnzungen                                                                */
/*  (Name, Datum, Beschreibung):    Kati Brodrecht, 13.06.2005, automatische Be-    */
/*                                  stimmung und Ausgabe eines Missing Pattern, so- */
/*                                  fern fehlende Werte im Datensatz vorhanden sind */
/*                                                                                  */
/************************************************************************************/
/*

 TITLE
 -----
 missdescription.mac.sas

 SAS Makro zur Deskription der Missing-value- Situation im angegebenen Datensatz

 BACKGROUND
 ----------
 
 Bevor der Analyse eines Datensatzes sollte dieser stets deskriptiv betrachtet
 werden, um die Variablenstruktur, die Fallzahl, die missing- value- Situation,
 die Eigenschaften jeder Variable sowie den Datensatz allgemein kennenzulernen.

 Missing Values:

 Ein groes Problem bei der Analyse von Datenstze, u.a. in den meist umfang-
 reichen Datenstzen in der Rehabilitationsforschung ergibt sich, wenn viele 
 Messwerte im Datensatz fehlen. Das bliche Vorgehen z.B. in der Regressions-
 analyse ist das der Complete Case Analyse. Bei dieser Analyse werden nur die 
 Beobachtungen zur Modellierung herangezogen, die in allen fr die Modellierung 
 angegebenen Variablen keine fehlenden Werte haben. Viele weitere multivariate
 Analysemethoden beruhen auf dem gleichen Prinzip. Durch dieses Vorgehen kann 
 der Datensatz, der dann fr die Auswertung zur Verfgung steht, sehr viel kleiner 
 sein als der Ausgangsdatensatz, auch wenn in jeder Variable nur einige wenige 
 fehlende Werte auftreten. Noch problematischer ist es, wenn in einer inhaltlich 
 interessanten Variable sehr viele fehlende Werte vorkommen. Dies kann passieren, 
 wenn die Messung der Variablen von anderen Variablen abhngt. Die Verkleinerung
 des Ausgangsdatensatzes ist zum einen ein Problem fr die statistische Power,
 kann aber auch eine starke Verzerrung der Ergebnisse verursachen, wenn die
 fehlenden Werte nur von Beobachtungen einer speziellen Subgruppe sind und so
 die Reprsentativitt und Verallgemeinerbarkeit der Ergebnisse in Frage gestellt
 werden muss. 

 Deskription der fehlenden Werte:
 
 Deshalb sollte zuerst die Struktur der fehlenden Werte untersucht werden. Dazu 
 wird deskriptiv erst einmal festgestellt, wie viele Werte in jeder der zu 
 untersuchenden Variablen fehlen. Dabei sollten die Variablen identifiziert 
 werden, die nur selten beobachtet wurden. Eine inhaltliche berprfung der 
 Grnde des Fehlens sollte sich sofort anschlieen. Gleichzeitig kann es 
 vorkommen, dass von bestimmten Beobachtungen nur sehr wenig Information vor-
 liegt. Deshalb wird ebenfalls die Anzahl fehlender Werte pro Beobachtung 
 analysiert. 

 Deskription der Variablen:

 Die fr die jeweilige sptere Auswertung in Frage kommenden Variablen sollten 
 vor der Analyse genauer betrachtet werden, da deren Eigenschaften spter 
 z.B. whrend der Modellierung eine wichtige Rolle spielen. Zum Beispiel ist es 
 fr einige Analysen wesentlich, die Verteilung der stetigen Variablen zu kennen. 
 Dazu gehrt auch eine grafische Darstellung z.B. ber ein Histogramm sowie die 
 Angabe verschiedener Perzentilwerte. Unter anderem ist es auch wichtig, schwach 
 besetzte Kategorien zu entdecken, um eventuell durch Zusammenfassungen von 
 Merkmalsausprgungen die Fallzahlen pro Kategorie nicht allzu klein werden zu 
 lassen. 

 DESCRIPTION
 -----------

 Im Makro sind die angesprochenen deskriptiven Auswertungen bzgl. der fehlenden 
 Werte bzw. der Beschreibung der Modellvariablen folgendermaen implementiert 
 worden:
 - Missing Values:  Ausgabe der Anzahl fehlender Werte fr jede im Modellaufruf 
                    angegebene Variable. Auerdem wird optional fr jede 
                    Beobachtung die Anzahl fehlender Werte ber alle diese 
                    Variablen bestimmt, ausgegeben werden allerdings nach     
                    Benutzerangabe nur die x Beobachtungen mit den hufigsten 
                    fehlenden Werten. Diese Berechnung ist sehr zeitaufwendig 
                    und sollte nur bei Bedarf angefordert werden.
 - Deskription der Variablen: Fr die kategoriellen Variablen werden die absoluten  
					Hufigkeiten je Ausprgung bestimmt. Fr die stetigen Variablen 
					wird mittels PROC UNIVARIATE die Fallzahl, Mittelwert, 
                    Standardabweichung, Minimum, 20%-, 40%-, Median, 60%-, 
                    80%-Perzentil und Maximum fr das Gesamtkollektiv bestimmt. 
					Auerdem werden die Beobachtungen nach den Quintilen des 
					Gesamtkollektivs in 5 Klassen eingeteilt und eine 
					entsprechende Tabelle ausgegeben. Diese Auswertung wird durch-
                    gefhrt, falls die Anzahl der Merkmalsausprgungen grer 
                    ist als eine im Makro angegebene Anzahl verschiedener 
                    Ausprgungen der Variable. Ansonsten wird ebenfalls eine 
                    Tabelle ausgegeben. Fr die stetigen Variablen kann 
                    optional jeweils ein Histogramm in einem zu whlenden 
                    Grafikformat ausgegeben und abgespeichert werden. 


 LITERATURE
 ----------
 - SAS Online Doc zu den PROC FREQ und PROC UNIVARIATE
 - Ziegler C (2003): Ein SAS-Makropaket zur Entwicklung und Validierung von 
					 Prognosemodellen auf Basis der logistischen Regression. 
					 Diplomarbeit, FH Ulm 

 SYNTAX
 ------
 [Makroaufruf und Beschreibung der zu bergebenden Parameter]

 %PM_DESCRIPTION(	
   *data			= ,				Datensatz
   *xvar			= ,				stetige Variable(n)
	xvar_cat		= 10,			Kriterium fr die # Kategorien der stetigen Variablen
   *cvar			= ,				kategorielle Variable(n)
    miss			= 0,			Missing Values ausschliessen (ja/ nein)
    extreme			= 0,            Beobachtungen mit hufigsten Missings ausgeben (ja/nein)
    most_extreme	= 5,			Anzahl Beobachtungen mit hufigsten Missings
   *obs_id			= ,				Variablenname der Beobachtungsnummern im Datensatz
    graphics		= 1,			Ausgabe der Grafiken (ja/nein)
	ext				= tiffp,		Format fr Grafikdateien
    path			= 'c:\temp',	Pfad fr Grafikausgaben
   *macro_path		=				Pfad zur Makrosammlung
  );

  Mit (*) gekennzeichnete Parameter sind beim Makroaufruf zwingend erforderlich.
  Bereits gesetzte Parameter stellen die jeweilige Standardeinstellung dar.

   data:			Zur Angabe des zu analysierenden Datensatzes. 
					Wurde beispielsweise vor dem Aufruf des Makros ein Datensatz mit
					dem Namen "dat300" angelegt, lautet der korrekte Aufruf: 
					'data=dat300'. Zustzlich kann auch die dazugehrige SAS- Library
					bergeben werden (Beispiel: 'data=work.dat300'). 
					Die bergabe dieses Parameters ist zwingend erforderlich.

   xvar:			Dieser Parameter ist zur Angabe aller stetigen Variablen. Werden 
					mehrere stetige oder quasi- stetige Variablen bergeben, so sollten 
					diese durch ein Blank (Leerzeichen) getrennt werden. Beispiel: 
					'xvar= alter bmi'.

   xvar_cat:		Der Parameter 'xvar_cat' ermglicht die bergabe eines Kriteriums,
					ab wie vielen Ausprgungen (Kategorien) die unter dem Parameter
					'xvar' angegebenen stetigen Variablen als echt stetig betrachtet
					werden sollen. Die Standardeinstellung ist 'xvar_cat=10'.
					Besitzt eine stetige Variable weniger Ausprgungen als die unter
					'xvar_cat' angegebene Zahl, so wird diese aufgrund weniger 
					verschiedener Ausprgungen als kategoriell betrachtet. Dies hat
					zur Folge, dass die Deskription mittels PROC FREQ
					(wie bei kategoriellen Variablen, Parameter 'cvar') durchgefhrt
					wird. Erfllt die stetige Variable das Kriterium, so wird mittels
					PROC UNIVARIATE gearbeitet und zustzlich ein Histogramm ausgegeben
					(falls Parameter 'graphics=1' gesetzt ist).

   cvar:			"cvar" steht fr "categorial variables" und ist fr die bergabe
					aller kategorischen sowie aller dichotomen Variablen 
					verantwortlich. Werden mehrere kategorischen 
					Variablen bergeben sollten diese getrennt durch ein Leerzeichen
					bergeben werden (Beispiel:'cvar= diagnose msnr'). 

   miss:			Dieser Parameter legt fest, ob Beobachtungen mit fehlenden 
					Werten (missing values) in den unter 'xvar', 'cvar' und 
					'resp_var' bergebenen Variablen vor der Deskription 
					aus dem Datensatz eliminiert werden sollen (0) oder nicht (1). 
					Der Standardwert dieses Parameters ist 0 
					(Complete Case- Analyse). Hierbei ist zu beachten, dass 
					fehlende Werte im Datensatz durch einen Punkt (.) reprsentiert 
					sein mssen. Die Einschrnkung auf den Complete Case-Datensatz gilt 
                    "nur" fr die Deskription der Variablen, die Beschreibung 
                    der fehlenden Werte wird nicht eingeschrnkt.

   extreme:			Auswahl, ob die Beobachtungen mit den hufigsten Missing values 
                    bestimmt und ausgegeben werden sollen (Standard=0). Bedingt eine 
                    lngere Laufzeit des Makros!
 
   mostextreme:		Parameter zur Angabe der Anzahl auszugegebener Beobachtungen mit 
					der hchsten Zahl an fehlenden Werten. Standard=5. Wir dem 
					Parameter beispielsweise der Wert 10 bergeben, so werden im 
					Output die 10 Beobachtungen mit den meisten fehlenden Werten 
					ausgegeben (mit entsprechender Anzahl).

   obs_id:			Dieser zwingend erforderliche Parameter dient der bergabe des
   					Variablennamens der Beobachtungsnummern im angegebenen Datensatz.
   					Diese Variable wird bentigt, um Beobachtungen eindeutig zu 
					identifizieren. Diese Variable darf keine missing values enthalten
					und mu darber hinaus Beobachtungen eindeutig identifizieren.

   graphics:		Der optionale Parameter 'graphics' dient der Auswahl, ob 
					zustzliche Grafiken erzeugt werden sollen (1) oder nicht (0/ 
					Standard). Wird der Parameter auf den Wert 1 gesetzt, so wird
					fr jede unter 'xvar' angegeben stetige Variable, welche das 
					mittels 'xvar_cat' bergebene Kriterium erfllt, eine Grafik 
					(Histogramm zur Verteilungsprfung) in das unter dem Parameter 
                    'path' angegebene Verzeichnis ausgegeben.
					Das Grafikformat wird ber den Parameter 'ext=' bestimmt.

   ext:				Der Parameter 'ext' (Extension) dient der Bestimmung des 
					Grafikformates, falls Grafiken angefordert wurden (Parameter
					'graphics=1'). Die Standardeinstellung ist'ext=tiffp', das
					heit, dass alle Grafiken im TIFF- Format ausgegeben werden.
					Mgliche Angaben sind:
					GIF 			GIF format 										
				 	GIF160 			GIF format (160 by 120 pixels) 					
					GIF260 			GIF format (260 by 195 pixels) 					
					GIF373 			GIF format (373 by 280 pixels) 					
					GIF570 			GIF format (570 by 480 pixels) 					
					GIF733 			GIF format (733 by 550 pixels) 					
				   	GIFANIM 		GIF animation driver 							
			 	 	IMGGIF 			GIF format 										
				 	IMGJPEG 		JPEG format 									
				 	JPEG 			JPEG format 									
				 	TIFFP 			TIFF format 									
				 	IMGTIFB 		TIFF format (black and white, uncompressed)		
				 	IMGTIFG3 		TIFF format (black and white, CCITT Group 3) 	
				 	IMGTIFG4 		TIFF format (black and white, CCITT Group 4) 	
				 	IMGTIFP 		TIFF format (256 colors, uncompressed) 			
				 	BMP 			BMP format  (256 colors) (Windows host driver) 
				 	BMP20 			BMP 2.0 file format (256 colors) 				
				 	IMGBMP			BMP format 										
				 	IMGPAINT 		BMP format  (MS Windows Paintbrush) 				
				 	WMF 			Windows Metafile format 						
				 	EMFDRV 			Windows Enhanced Metafile format 				
				 	PDF 			PDF format 										
				 	PDFC 			PDF format  (color) 		

   path:			Dieser Parameter erlaubt die benutzerspezifische bergabe des
					Zielverzeichnisses fr die erzeugten Grafiken, falls der
					Parameter 'graphics' auf den Wert 1 gesetzt wurde. Die
					Standardeinstellung ist path='c:\temp'.
					
   macro_path: 		Aufgrund der Tatsache, dass innerhalb des Makros diverse 
					Untermakros aufgerufen werden, ist es zwingend ntig, den Pfad 
					zu dem Verzeichnis, wo die komplette Makrosammlung abgelegt ist,
					anzugeben. Der Pfad selber muss in Hochkommata eingeschlossen 
					sein. Beispiel: 'f:\makros'. 

  VALIDATION
  ----------

  Validiert durch: Stephanie Heiss
 				   University of Ulm 
                   Department of Biometry and Medical Documentation
                   89075 Ulm
  
  Validiert am:    28. Oktober 2003

  Validierungsprogramm:       MISSDESCRIPTION_VAL.SAS
  Validierungsdokumentation:  MISSDESCRIPTION_VAL.DOC
   
  Validierungsvorgehen:
  - Das Testen der Makros wird durch eine von der Programmierung unabhngige 
    Person durchgefhrt.
  - Fr die Durchfhrung der Validierung wird auf einen in der Literatur 
    genutzten Datensatz zurckgegriffen:  LBW (Low Birth Weight), beschrieben
    in Hosmer, Lemeshow: Applied logistic regression [hosmer2000]
  - Der Datensatz wird fr verschiedene Testlufe verndert bzw. erweitert, 
    damit "die Testdaten neben den normalen Angaben auch Grenzwerte, ungltige
    Angaben, Fehlerbedingungen und Sonderflle beinhalten." [beiwanger2003]
  - Fehlerkontrolle:
    Hierbei werden nur die im LOG-Fenster auftretenden Fehler bzw. Warnungen 
    und offensichtlichen Fehler im OUTPUT betrachtet. Fr jeden Parameter des 
    Makros werden mehrere Durchlufe mit verschiedenen Wertangaben durchgefhrt, 
    wobei auch ungltige Werte angegeben werden, um das Verhalten des Makros
    bei fehlerhaften Aufrufen zu untersuchen. Gefundene Fehler werden 
    dokumentiert und dem Programmierer zur Verfgung gestellt. Nach der Korrektur 
    aller Fehler werden die Testlufe wiederholt, um sicherzustellen, dass 
    wirklich alle Fehler abgefangen wurden und dass keine neuen Fehler entstanden 
    sind.
  - Ergebniskontrolle:
    Um die Richtigkeit der Ergebnisse zu berprfen, werden soweit mglich, die
    Rechenbeispiele und bungsaufgaben aus Hosmer/Lemeshow durch vergleichbare
    Makroaufrufe nachgerechnet. Die Ergebnisse werden denen aus dem Begleitbuch 
    Hosmer/Lemeshow/Cook [hosmer2001] gegenbergestellt und direkt verglichen.
  - Alle Validierungen werden dokumentiert.

  Literatur: 
  - Beiwanger S (2003): Validierung von SAS-Makros. Diplomarbeit, FH Ulm
  - Hosmer DW, Lemeshow S (2000): Applied logistic regression 2nd Ed. 
           J. Wiley & Sons, New York
  - Hosmer DW, Lemeshow S, Cook ED (2001): Solutions manual to accompany
           applied logistic regression 2nd. Ed. J. Wiley & Sons, New York



  AUTHOR
  ------
  R. Muche / C. Ring / C. Ziegler
  University of Ulm
  Department of Biometry and Medical Documentation
  89075 Ulm

  The author does not overtake any responsibility for the correctness
  of results. It must not be modified without removing the heading lines 
  containing the authors name.

----------------------------------------------------------------------------------*/

%MACRO MISSDESCRIPTION ( 	data			= ,
							xvar			= ,
							xvar_cat		= 10,
							cvar			= ,
							miss			= 0,
							extreme			= 0,
							mostextreme		= 5,
							obs_id			= ,
							graphics		= 1,
							ext				= tiffp,
							path			= 'c:\temp',
							macro_path		= 	
			          );

%LET missx=&miss;
%LET misspatt=0;


/*Prfungen / Referenzierungen / Standardbelegungen*/

/*Wichtigster Parameter 'macro_path' berprfen*/
/*bergabe erfolgt?*/

%IF &macro_path= %THEN %DO;

	%WINDOW error color=grey      
	 #5  @28 'Fehler!' attr=highlight
              color=blue
     #7  @28 "Dem zwingend erforderlichen Parameter 'macro_path' wurde kein Wert" 
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm welches berprft, ob dem Parameter 'macro_path' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/

/*Anzahl der Zeichen im String 'macro_path' bestimmen:*/

%LET path_length=%LENGTH(&macro_path);

%LET infile=&macro_path;

%IF %BQUOTE(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %BQUOTE(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %BQUOTE(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

/*falls die Syntax nicht stimmt, Anzeige eines Hinweisfensters*/

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

	%WINDOW error color=grey                                                    
	 #5  @28 'Fehler!' attr=highlight    
              color=blue
     #7  @28 "Dem Parameter 'macro_path' wurde ein syntaktisch falscher Wert"
	 #8  @28 "bergeben- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
	%DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das unter 'macro_path' angegebene Verzeichnis existiert*/

%LET dir=&macro_path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

	%WINDOW error color=grey                                                   
	 #5  @28 'Fehler!' attr=highlight 
              color=blue
     #7  @28 "Das unter dem Parameter 'macro_path' angegebene Verzeichnis existiert"
	 #8  @28 "nicht- Bentigte Untermakros knnen nicht referenziert werden"
     #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';
    %DISPLAY error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Falls bestimmte Parameter ohne Wert bergeben wurden, diese mit Standard belegen*/

%IF &xvar_cat= %THEN %DO;
	%LET xvar_cat=10;
%END;

%IF &mostextreme= %THEN %DO;
	%LET mostextreme=5;
%END;

%IF &graphics= %THEN %DO;
	%LET graphics=1;
%END;

%IF &path= %THEN %DO;
	%LET path='c:\temp';
%END;

%IF &ext= %THEN %DO;
	%LET ext=tiffp;
%END;

%IF &extreme= %THEN %DO;
	%LET extreme=0;
%END;

%IF &missx= %THEN %DO;
	%LET missx=0;
%END;

/*Externe Makros, die innerhalb des Makros 'PM_DESCRIPTION.MAC.SAS'*/
/*verwendet werden (Untermakros) einbinden (%include)*/

FILENAME y &macro_path;

%INCLUDE y ('numberobs.mac.sas');
%INCLUDE y ('exist.mac.sas');
%INCLUDE y ('exist_var.mac.sas');
%INCLUDE y ('numchar.mac.sas');

/*globale Variable des Untermakros 'numberobs.mac.sas'*/

%GLOBAL nnnn1;

/*berprfung, ob Datenquelle (Parameter 'data=') angegeben wurde*/

%IF &data= %THEN %DO;

/*wenn nicht, Anzeige eines Fensters mit Fehlermeldung/ Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Dem Parameter 'data' (Datensatz)"  
		   #8  @28 "wurde kein Wert bergeben!"   
		   #9  @28 "Dieser Parameter ist jedoch zwingend erforderlich."  
           #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob berhaupt Variablen (Parameter 'xvar=' / 'cvar=') angegeben wurden*/

%IF &xvar= AND &cvar= %THEN %DO;

/*Falls keine Variablen angegeben wurden,*/
/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Den Parametern 'xvar' und 'cvar' wurden keine Variablen bergeben!"
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob dem zwingend erforderlichen Parameter 'obs_id' ein Wert bergeben wurde*/

%IF &obs_id= %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Dem zwingend erforderlichen Parameter 'obs_id' wurde kein Wert" 
		   #8  @28 "bergeben! Dieser bentigt den Variablenname der Beobachtungs-"
		   #9  @28 "nummern in dem zu untersuchenden Datensatz"
           #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfen, ob der angegebene Datensatz (Parameter 'data=') auch wirklich existiert*/
/*(mit Hilfe des Hilfs-/Untermakros 'exist.mac.sas')*/

%EXIST(dsn=&data);

/*falls Datensatz nicht existiert, Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Der unter 'data' angegebene Datensatz existiert nicht!"                                                                                        
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error; 

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Prfung, ob alle bergebenen Variablen numerisch sind*/

/*Erzeugung eines temporren Zwischendatensatzes, welcher lediglich die*/
/*bergebenen Variablen enthlt*/

DATA test3;
	SET &data;
	KEEP &xvar &cvar &obs_id;
RUN;

/*mittels der Prozedur PROC CONTENTS kann der Typ einer Variablen bestimmt werden*/

/*Unterdrckung der Ausgabe von PROC CONTENTS*/

PROC CONTENTS DATA=test3 NOPRINT OUT=testcontent;
RUN;

/*lediglich die Variable 'type' der Ergebnisdatei behalten,*/
/*da nur diese von Interesse ist*/

DATA testcontent;
	SET testcontent;
	KEEP type;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=testcontent);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test4=&nnnn1;

/*Prfung ob Variablen vom Typ 'character' vorhanden sind*/

DATA test4;
	SET testcontent;
	IF type=2 THEN DELETE;
RUN;

/*Anzahl der Beobachtungen in der Ergebnisdatei mit Hilfe des Untermakros*/
/*'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test4);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test5=&nnnn1;

/*falls Variablen bergeben wurden, die nicht numerisch sind --> Fehlermeldung*/

%IF &test4 ne &test5 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Der unter 'data' angegebene Datensatz enthlt Variablen" 
		   #8  @28 "vom Typ 'character'. Diese mssen jedoch zwingend"	
		   #9  @28 "vom Typ 'numeric' sein!"	  
           #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
     %display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test4 testcontent test3;
RUN;
QUIT;

/*berprfung, ob alle bergebenen Variablen im bergebenen Datensatz*/
/*(Parameter 'data' vorhanden sind)*/

/*Zusammenfassung aller Variablen in einer neuen Makro- Variablen*/

%LET all_variables=&xvar &cvar &obs_id;

/*Prfung erfolgt mit Hilfe des Untermakros 'exist_var.mac.sas'*/

%EXIST_VAR(dsn=&data, var=&all_variables);

/*falls irgendwelche Variablen nicht im angegebenen Datensatz existieren,*/
/*Programm beenden*/

%IF &exist='NO' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Eine oder mehrere bergebene Variable(n)"                                                                                        
           #8  @28 "konnte nicht im angegebenen Datensatz gefunden werden"
		   #9  @28 "oder es wurde eine oder mehrere Variable(n) doppelt angegeben!"
		   #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob bei bestimmten Parametern anstatt numerischen Werten*/
/*character- Werte bergeben wurden*/

/*berprfung erfolgt mit Hilfe des Untermakros 'numchar.mac.sas'*/

%NUMCHAR(parameterliste=&mostextreme &xvar_cat &missx &graphics &extreme);

%IF &numcharok='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Einem oder mehreren Parameter(n) wurde(n) anstatt eines" 
		   #8  @28 "numerischen Wertes flschlicherweise ein alphanumerischer Wert bergeben!"	 
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Parameter, bei denen keine negativen Werte erlaubt sind, berprfen*/

%IF &xvar_cat <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Einem oder mehreren Parameter(n) wurde(n) anstatt eines" 
		   #8  @28 "positiven Wertes flschlicherweise ein negativer Wert oder 0 bergeben!"	 
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;


%IF &mostextreme <=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Einem oder mehreren Parameter(n) wurde(n) anstatt eines" 
		   #8  @28 "positiven Wertes flschlicherweise ein negativer Wert oder 0 bergeben!"	 
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*berprfung, ob der Parameter 'graphics=' 0 oder 1 ist*/

%IF &graphics ne 1  %THEN %DO;
	%IF &graphics ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Parameter 'graphics' <> 0 oder 1!"                                                                                        
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
		%display error;

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob der Parameter 'miss=' 0 oder 1 ist*/

%IF &missx ne 1  %THEN %DO;
	%IF &missx ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Parameter 'miss' <> 0 oder 1!"                                                                                        
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
		%display error;

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob der Parameter 'extreme=' 0 oder 1 ist*/

%IF &extreme ne 1  %THEN %DO;
	%IF &extreme ne 0  %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

		%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Parameter 'extreme' <> 0 oder 1!"                                                                                        
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
		%display error;

/*Sprungmarke im Fehlerfall*/

		%GOTO ende;

	%END;
%END;

/*berprfung, ob das Ausgabebildformat (Parameter 'ext=' korrekt ist:*/
/*(nur wenn Grafiken erwnscht sind (Parameter 'graphics=1')*/

%IF &graphics=1 %THEN %DO;

/*Zusammenfassung aller mglichen Formate in einer Makrovariablen*/

%LET ext_list=GIF GIF160 GIF260 GIF373 GIF570 GIF733 GIFANIM IMGGIF IMGJPEG JPEG TIFFP IMGTIFB IMGTIFG3 IMGTIFG4 IMGTIFP BMP BMP20 IMGBMP IMGPAINT WMF EMFDRV PDF PDFC; 

/*Schleife ber alle mglichen Formate*/

%DO h=1 %TO 23;

 %LET extension=%QSCAN(&ext_list,&h,%STR( ));

 	%IF &extension = %UPCASE(&ext) %THEN %DO;

 		%LET ext_check='OK';

/*Sprungmarke aus der Schleife, falls korrektes Grafikformat gefunden wurde*/

		%GOTO endext;

 	%END;
	%ELSE %DO;

/*Falls das bergebene Grafikformat nicht gefunden wurde:*/

		%LET ext_check='NOK';

	%END;

%END;

%endext:

%IF &ext_check ='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Dem Parameter 'ext' wurde ein falscher Wert bergeben!" 
		   #8  @28 "Mgliche Dateiendungen fr Grafikformate sind:"
		   #9  @28 "GIF GIF160 GIF260 GIF373 GIF570 GIF733 GIFANIM IMGGIF IMGJPEG"
		   #10 @28 "JPEG TIFFP IMGTIFB IMGTIFG3 IMGTIFG4 IMGTIFP BMP BMP20 IMGBMP"
           #11 @28 "IMGPAINT WMF EMFDRV PDF PDFC"
		   #15 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Hilfsprogramm, welches berprft, ob dem Parameter 'path=' ein korrekter Pfad*/
/*bergeben wurde (mit einfachen Anfhrungszeichen vorne und hinten)*/
/*berprfung erfolgt ebenfalls nur, falls Grafiken erwnscht sind*/
/*(Parameter 'graphics=1')*/

/*Anzahl der Zeichen im String 'path' bestimmen:*/

%LET path_length=%LENGTH(&path);

%LET infile=&path;

%IF %BQUOTE(&infile) NE %THEN %DO;
     %LET char1 = %BQUOTE(%SUBSTR(&infile,1,1));
      %IF %BQUOTE(&char1) ne %STR(%')
         %THEN %LET command='NOK';
         %ELSE %LET command='OK';

	 %LET char2 = %BQUOTE(%SUBSTR(&infile,&path_length,1));
      %IF %BQUOTE(&char2) ne %STR(%')
         %THEN %LET command2='NOK';
         %ELSE %LET command2='OK';
%END;

%IF &command='NOK' OR &command2='NOK' %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Dem Parameter 'path' wurde ein syntaktisch falscher Wert" 
		   #8  @28 "bergeben- Grafikdateien knnen nicht abgespeichert werden"	 
           #12 @28 'Press ENTER to continue';                                                                                           
     %display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Test, ob das angegebene Zielverzeichnis fr die Grafiken existiert*/

%LET dir=&path;

%LET filrf=testdir;
%LET rc=%SYSFUNC(filename(filrf,&dir));
%LET did=%SYSFUNC(dopen(&filrf));

%IF &did=0 %THEN %DO;
       %PUT Directory does not exist;
%END;
%ELSE %DO;
       %PUT Directory exists;
       %LET rc=%SYSFUNC(dclose(&did));
%END;

%IF &did=0 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Das unter dem Parameter 'path' angegebene Verzeichnis existiert" 
		   #8  @28 "nicht- Grafikdateien knnen nicht gespeichert werden"	 
           #12 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
    %display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*Ende '%IF &graphics=1'*/
%END;

/*prfen, ob Variable 'obs_id' missings enthlt*/
/*(diese mu eindeutig sein)*/

/*Erzeugung eines temporren Zwischendatensatzes, der nur diese Variable enthlt*/

DATA test;
	SET &data;
	KEEP=&obs_id;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test1=&nnnn1;

/*Elimination aller etwaiger missing values der Variable 'obs_id'*/

DATA test2;
	SET test;
	IF &obs_id=. THEN DELETE;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=test2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET test2=&nnnn1;

%IF &test1 ne &test2 %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

	%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Die Identifikationsvariable der Beobachtungen, welche unter" 
		   #8  @28 "dem Parameter 'obs_id' angegeben wurde enthlt missing values!"	
		   #9  @28 "Beobachtungen knnen nicht eindeutig identifiziert werden."	  
           #13 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
 	%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE test test2;
RUN;
QUIT;

/*prfen ob Variable 'obs_id' eindeutig ist*/

/*Output von PROC FREQ unterdrcken*/

ODS LISTING CLOSE;

/*Ergebnis in eine ODS- Datei ausgeben*/

ODS OUTPUT OneWayFreqs=freqs;
	PROC FREQ DATA=&data;
		TABLES &obs_id;
	RUN;

/*ODS- Output schliessen*/

ODS OUTPUT CLOSE;

/*Output wieder aktivieren*/

ODS LISTING;

/*Variable 'frequency' in der Ergebnisdatei enthlt die gesuchte Information*/

DATA freqs;
	SET freqs;
	KEEP frequency;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs2=&nnnn1;

/*Jede Beobachtungsnummer darf lediglich einmal vorkommen (Eindeutigkeit)*/

DATA freqs;
	SET freqs;
	IF frequency =1;
RUN;

/*# Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen*/

%NUMBEROBS(datensatz=freqs);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET freqs=&nnnn1;

%IF &freqs2 ne &freqs %THEN %DO;

/*Anzeige eines Fensters mit entsprechender Fehlermeldung/ entsprechendem Hinweis*/

%window error color=grey                                                                                                                
           #5  @28 'Fehler!' attr=highlight                                                                                 
               color=blue                                                                                                                
           #7  @28 "Die unter 'obs_id' angegebene Beobachtungsidentifikationsnummer" 
		   #8  @28 "ist nicht eindeutig! Dies ist jedoch zwingend erforderlich."	
		   #9  @28 "Einfhrung einer neuen Identifikationsnummer: "	  
		   #10 @28 "DATA mydat; SET mydat; obs_id_neu=_N_; RUN;"	 
           #14 @28 'Bitte die ENTER- Taste drcken, um fortzufahren.';                                                                                           
%display error;

/*Sprungmarke im Fehlerfall*/

	%GOTO ende;

%END;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE freqs;
RUN;
QUIT;

/*ENDE Prfungen / Referenzierungen / Standardbelegungen*/

/*-------------------------------------------------------------------------------*/

/*Standardoutput ausgeben*/

%LET datensatz=&data;

/*Fallzahl mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=&data);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET fallzahl=&nnnn1;

/*Makro-kopf ausgeben*/

DATA standardoutput;
	makroinfo="";
RUN;

PROC PRINT DATA=standardoutput NOOBS LABEL HEADING=H WIDTH=UNIFORM;
	VAR   makroinfo;

    TITLE1 "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++";
	TITLE2 ;
	TITLE3 "Allgemeine Makroinformationen:           ";
	TITLE4 ;
	TITLE5 "Makroname:              &sysmacroname    ";
	TITLE6 "Analysierter Datensatz: &datensatz       ";
	TITLE7 "Fallzahl:               &fallzahl        ";
	TITLE8 "Datum:                  &sysday &sysdate ";
	TITLE9 "Start der SAS-Session:  &systime         ";
	
	LABEL makroinfo = '+++++++++++++++++++++++++++++++++++++++++++++++++++++++++';
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE standardoutput;
RUN;
QUIT;

/*ENDE Standardoutput*/

/*----------------------------------------------------------------------*/

/*Analyse der missing value- Situation im angegebenen Datensatz*/

/*zuerst, temporren Datensatz anlegen, der nur die bergebenen Variablen enthlt:*/

DATA _tmp_ (KEEP=&xvar &cvar &obs_id);
	SET &data;
RUN;

/*Anzahl der angegebenen kategoriellen Variablen ermitteln:*/

%GLOBAL c_count c_var;
%LET c_count=1;
%LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%DO %WHILE(&c_var ne);
     %LET c_count=%EVAL(&c_count+1);
     %LET c_var=%QSCAN(&cvar,&c_count,%STR( ));
%END;
%LET c_count=%EVAL(&c_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/

%PUT The cvar contains &c_count variables.;

/*Anzahl der angegebenen stetigen Variablen ermitteln:*/

%GLOBAL x_count x_var;
%LET x_count=1;
%LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%DO %WHILE(&x_var ne);
     %LET x_count=%EVAL(&x_count+1);
     %LET x_var=%QSCAN(&xvar,&x_count,%STR( ));
%END;
%LET x_count=%EVAL(&x_count-1);

/*Anzahl zum berblick im LOG- Fenster ausgeben*/
%PUT The xvar contains &x_count variables.;

/*Ermitteln, wieviel Prozent der Beobachtungen missing values enthalten:*/

/*# Beobachtungen im Originaldatensatz*/
/*mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=_tmp_);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET data_count =&nnnn1;

/*alle missing values entfernen*/

DATA nomiss;
	SET  _tmp_;

		%DO i=1 %TO &x_count;

			%LET xvar_liste=%SCAN(&xvar,&i,%STR( ));
			IF &xvar_liste=. THEN DELETE;

		%END;

		%DO i=1 %TO &c_count;

			%LET cvar_liste=%SCAN(&cvar,&i,%STR( ));
			IF &cvar_liste=. THEN DELETE;

		%END;
 
RUN;

/*#Beobachtungen im Datensatz ohne missing values*/
/*mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=nomiss);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET miss_count =&nnnn1;

/*ergnzt am 13.06.2005*/
/*Bestimmung des Missing Pattern mit Proc MI, wenn mind. 1 fehlender Wert
  vorhanden ist*/

%IF &miss_count < &data_count %THEN %DO;
  %LET misspatt=1;
%END;

%IF &misspatt=1 %THEN %DO;

  ODS LISTING CLOSE;
  ODS OUTPUT MissPattern=misspattern;

  PROC MI
    DATA=_tmp_
    ;
    VAR &xvar
        &cvar
    ; 
  RUN;

  ODS LISTING;
  ODS OUTPUT CLOSE;
%END;


/*Prozentzahl bestimmen*/

%LET factor=100;
%LET x=%SYSEVALF(&data_count/&factor);
%LET miss=%SYSEVALF(&data_count-&miss_count);
%LET percent_miss=%SYSEVALF(&miss/&x);

/*Prozentzahl auf eine Stellen nach dem Komma runden*/

DATA round;
	percent_miss=&percent_miss;
	call symput('percent_miss',trim(left(put(percent_miss,BEST5.))));
RUN;

/*Ausgabe im LOG- Fenster zur bersicht*/

%PUT &percent_miss;

/*Bestimmen, wie viele Beobachtungen je Variable missing values enthalten:*/

/*stetige Variablen (xvar)*/

%GLOBAL counter_xvar_ges;

%DO d=1 %TO &x_count;

%LET xvar_liste=%SCAN(&xvar,&d,%STR( ));

DATA tempo2 (KEEP= &xvar_liste);
	SET _tmp_;

	IF &xvar_liste ne '.' THEN DELETE;

RUN;

/*#Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=tempo2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET counter_xvar=&nnnn1;

%LET counter_xvar_ges =&counter_xvar_ges &counter_xvar;

%END;

/*kategorielle Variablen (cvar)*/

%GLOBAL counter_cvar_ges;

%DO a=1 %TO &c_count;

%LET cvar_liste=%SCAN(&cvar,&a,%STR( ));

DATA tempo3 (KEEP= &cvar_liste);
	SET _tmp_;

	IF &cvar_liste ne '.' THEN DELETE;

RUN;

/*#Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=tempo3);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET counter_cvar=&nnnn1;

%LET counter_cvar_ges =&counter_cvar_ges &counter_cvar;

%END;

/*Strings in Datensatz schreiben, damit dieser im Output Fenster ausgegeben*/
/*werden kann*/

/*(cvar)*/

	DATA var_uebersicht;
		variable		 ='                                                                ';
		count			 ='                                                                ';
		format percent 5.2;

		%DO m=1 %TO &c_count;

			variable	="%SCAN(&cvar,&m," ")";
			count		="%SCAN(&counter_cvar_ges,&m," ")";
			percent		="%SYSEVALF((%scan(&counter_cvar_ges,&m," "))/(&data_count/100))";
			output var_uebersicht;

		%END;

	RUN;

/*(xvar)*/

	DATA var_uebersicht2;
		variable2		 ='                                                                ';
		count			 ='                                                                ';
		format percent 5.2;

		%DO j=1 %TO &x_count;

			variable2	="%SCAN(&xvar,&j," ")";
			count		="%SCAN(&counter_xvar_ges,&j," ")";
			percent		="%SYSEVALF((%scan(&counter_xvar_ges,&j," "))/(&data_count/100))";
			output var_uebersicht2;

		%END;
	RUN;

/*Strings wieder zurcksetzen:*/

%LET varstring=;
%LET counter_cvar_ges=;
%LET counter_xvar_ges=;

/*Beobachtungen mit den meisten missing values bestimmen, falls Parameter 'extreme=1'*/

%IF &extreme=1 %THEN %DO;

/*neue Beobachtungsnummer einfhren sowie nur die bergebenen Variablen behalten*/

DATA misscount (KEEP=&xvar &cvar _newobs_ &obs_id);
	SET _tmp_;
	_newobs_=_N_;
RUN;

DATA final;
	SET misscount;
RUN;

/*#Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=misscount);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET c=&nnnn1;

%DO a=1 %TO &c;

/*jeweils alle Zeilen (Beobachtungen) bis auf eine lschen*/

	DATA miss;
		SET misscount;
		IF _newobs_ ne &a THEN DELETE;
	RUN;

/*transponieren*/

	PROC TRANSPOSE DATA= miss OUT=miss2;
	RUN;

/*berprfen, ob Variable _Label_ vorhanden*/

%EXIST_VAR(dsn=miss2, var=_LABEL_);

/*falls Variable _Label_ nicht vorhanden:*/

	%IF &exist='NO' %THEN %DO;

	DATA miss2 (DROP=_NAME_);
		SET miss2;
		IF COL1 ne '.' THEN DELETE;
	RUN;

	%END;

/*falls Variable _Label_ vorhanden:*/

	%ELSE %DO;

/*Ausgabe anpassen und alles bis auf missing values lschen*/

	DATA miss2 (DROP=_NAME_ _LABEL_);
		SET miss2;
		IF COL1 ne '.' THEN DELETE;
	RUN;

	%END;

/*missing values je Beobachtung zhlen (mit Hilfe des Untermakros 'numberobs.mac.sas')*/

	%NUMBEROBS(datensatz=miss2);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

	%LET m=&nnnn1;

/*Anzahl missing values je Beobachtung in Ergebnisdatei eintragen*/

	DATA final;
		SET final;
		IF _newobs_=&a THEN DO;
		misscount=&m;
		END;
	RUN;

/*LOG- Fenster lschen um berlauf zu vermeiden*/

	DM "clear log;";

%END;

/*Datei "final" nach 'misscount' sortieren und bis auf Beobachtungsnummer alles lschen*/

PROC SORT DATA=final OUT=sort_final;
	BY DESCENDING misscount ;
RUN;

DATA sort_final (KEEP=misscount _newobs_ &obs_id);
	SET sort_final;
	IF misscount=0 THEN DELETE;
RUN;

/*#Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS (datensatz=sort_final);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET countmiss=&nnnn1;

%END;

/*bersicht ber die Missing- Value- Situation im angegebenen Datensatz*/
/*im Output- Fenster ausgeben:*/

/*Falls kategorielle Variablen angegeben wurden*/

%IF &cvar ne %THEN %DO;

PROC PRINT DATA=var_uebersicht NOOBS LABEL;
	TITLE1 'bersicht ber die missing-value-Situation im angegebenen Datensatz';
	TITLE2 "%SYSEVALF(&data_count-&miss_count)(&percent_miss %) aller Beobachtungen enthalten fehlende Werte";
	TITLE3 "Anzahl Beobachtungen: &data_count";
	TITLE4 ;
	TITLE5 'Deskription der fehlenden Werte';
		VAR variable count percent;
			LABEL variable='kategorielle Variable';
			LABEL count='Anzahl missing values';
			LABEL percent='%';
RUN;

%END;

/*falls stetige Variable angegeben wurden*/

%IF &xvar ne %THEN %DO;

PROC PRINT DATA=var_uebersicht2 NOOBS LABEL;

/*falls kategorielle Variablen angegeben wurden, ist berschrift nicht ntig*/

	%IF &cvar ne %THEN %DO;

	TITLE1 ;
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
	TITLE5 ;

	%END;
	%ELSE %DO;

	TITLE1 'bersicht ber die missing-value-Situation im angegebenen Datensatz';
	TITLE2 "%SYSEVALF(&data_count-&miss_count) (&percent_miss %) aller Beobachtungen enthalten fehlende Werte";
	TITLE3 "Anzahl Beobachtungen: &data_count";
	TITLE4 ;
	TITLE5 'Deskription der fehlenden Werte';

	%END;

		VAR variable2 count percent;
			LABEL variable2='stetige Variable';
			LABEL count='Anzahl missing values';
			LABEL percent='%';
RUN;

%END;

/*falls Parameter 'extreme=1' gesetzt wurde, Ausgabe der Beobachtungen mit den meisten*/
/*missing values*/

%IF &extreme=1 %THEN %DO;

%IF &countmiss >0 %THEN %DO;

/*nur die maximalen Werte ausgeben (Parameter 'mostextreme=')*/

DATA sort_final;
	SET sort_final;
	__obs__=_N_;
	IF __obs__ > &mostextreme THEN DELETE;
RUN;

PROC PRINT DATA=sort_final NOOBS LABEL;
	TITLE1 'bersicht ber die missing value- Situation pro Beobachtung';
	TITLE2 "&countmiss Beobachtungen enthalten fehlende Werte";
	TITLE3 "Anzeige der &mostextreme Beobachtungen mit den meisten fehlenden Werten";
	TITLE4 ;
		VAR &obs_id misscount;
			LABEL &obs_id  ='Beobachtungsnummer';
			LABEL misscount='Anzahl fehlende Werte';
RUN;

%END;

/*Falls keine Beobachtungen missing values enthalten:*/

%ELSE %DO;

DATA null3;
	status="Keine Beobachtung enthlt missing values!";
RUN;

PROC PRINT DATA=null3 NOOBS LABEL;
	TITLE1 'bersicht ber die missing value- Situation pro Beobachtung';
	TITLE2 ;
	TITLE3 ;
	TITLE4 ;
		VAR status;
			LABEL status='# Beobachtungen mit missing values:';
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE null3;
RUN;
QUIT;

%END;

%END;

/*Falls Auswertung ohne missing values erwnscht:*/

%IF &missx=0 %THEN %DO;

%LET data=nomiss;

%END;
%ELSE %DO;

PROC DATASETS NOLIST;
	DELETE nomiss;
RUN;
QUIT;

%END;

/*Temporre Zwischendatenstze lschen*/

PROC DATASETS NOLIST;
	DELETE _tmp_  round var_uebersicht2 var_uebersicht

%IF &extreme=1 %THEN %DO;

		   misscount miss2 miss final sort_final 

%END;
;

%IF &xvar ne %THEN %DO;

	DELETE tempo2;

%END;

%IF &cvar ne %THEN %DO;

	DELETE tempo3;

%END;

RUN;
QUIT;

/*ergnzt am 13.06.2005/20.06.2005*/
/*Ausgabe Missing Pattern*/

%IF &misspatt=1 %THEN %DO;

  DATA misspattern (DROP=&xvar &cvar);
    SET misspattern;
  RUN;

  PROC PRINT DATA=misspattern NOOBS LABEL;
    TITLE1 'Missing Data Patterns';
    TITLE2;
    TITLE3 "'X' = Wert vorhanden / '.' = missing / 'O' = missing, wird nicht ersetzt";
  RUN;

  PROC DATASETS NOLIST;
    DELETE misspattern;
  RUN;
  QUIT;
%END;

/*----------------------------------------------------------------*/

/*Deskription, Teil 2*/

/*kategorielle Variablen*/

/*berschrift ausgeben*/
	
DATA null;
	Datensatz="";
RUN;

PROC PRINT DATA=null NOOBS LABEL;
	TITLE1 'Deskription aller bergebenen Variablen';
	VAR Datensatz;
	LABEL Datensatz='---------------------------------------';
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE null;
RUN;

/*Ausgabe einer entsprechenden berschrift,*/
/*falls kategorielle Variablen bergeben wurden*/

%IF &cvar ne %THEN %DO;

DATA null;
	Analyse="kategorielle Variablen";
RUN;

PROC PRINT DATA=null NOOBS;
	TITLE1 ;
	TITLE2 ;
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE null;
RUN;
QUIT;

%END;

/*PROC FREQ's ber alle kategoriellen Variablen*/

%DO i=1 %TO &c_count;

    %LET var=%SCAN(&cvar,&i,%STR( ));

		PROC FREQ DATA=&data;
			TITLE;
			TABLES &var / NOROW NOPERCENT;
		RUN;

	%LET var=;

%END;

/*stetige Variablen*/

DATA null;
	Analyse="stetige Variablen";
RUN;

PROC PRINT DATA=null NOOBS;
	TITLE1 ;
	TITLE2 ;
RUN;

/*temporr angelegte Zwischendatei lschen*/

PROC DATASETS NOLIST;
	DELETE null;
RUN;
QUIT;

%DO i=1 %TO &x_count;

%LET var=%SCAN(&xvar,&i,%STR( ));

/*-----------------------------------*/

/*berprfung auf echte Stetigkeit*/

/*zuerst: # Ausprgungen je stetiger Variable bestimmen*/

PROC FREQ DATA=&data NOPRINT;
	TABLES &var /  SPARSE OUT=cat;
RUN;

/*Ausgabedatensatz von PROC FREQ anpassen*/

DATA cat (DROP=percent count);
	SET cat;
RUN;

/*#Beobachtungen mit Hilfe des Untermakros 'numberobs.mac.sas' bestimmen:*/

%NUMBEROBS(datensatz=cat);

/*Ergebnis aus 'numberobs.mac.sas' als neue Makro- Variable speichern*/

%LET cat_count=&nnnn1;

%IF &cat_count <&xvar_cat %THEN %DO;

/*stetige Variable wird aufgrund weniger*/
/*verschiedener Ausprgungen als kategoriell betrachtet*/
/*---> PROC FREQ, wie bei kategoriellen Variablen*/

/*entsprechende berschrift ausgeben*/

DATA null;
	Analyse="stetige Variable &var, kategoriell betrachtet";
	Auspraegungen="&cat_count";
RUN;

PROC PRINT DATA=null NOOBS;
	TITLE1 ;
	TITLE2 ;
RUN;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE null cat;
RUN;
QUIT;

PROC FREQ DATA=&data;
	TITLE;
	TABLES &var / NOROW NOPERCENT;
RUN;

%END;

%ELSE %DO;

/*-----------------------------------*/

/*PROC UNIVARIATE*/

PROC UNIVARIATE DATA=&data NOPRINT;
	VAR &var;
	OUTPUT OUT=&var N=N MEAN=MEAN STD=STD MIN=MIN pctlpre=P_ pctlpts=20 to 80 by 20 
					MAX=MAX MEDIAN=MEDIAN;
RUN;

PROC PRINT DATA=&var LABEL NOOBS;
	TITLE "The UNIVARIATE Procedure / Analysis Variable : &var";
	VAR  N MEAN STD MIN P_20 P_40 MEDIAN P_60 P_80 MAX;
	LABEL 		  n			="N Obs"
				  mean		="Mean"
				  std		="Std Dev"
				  min		="Minimum"
		     	  P_20		="20th Pctl"
				  P_40		="40th Pctl"
				  MEDIAN	="Median" 
				  P_60		="60th Pctl"
				  P_80		="80th Pctl"
				  MAX		="Maximum"
		;
RUN;

/*falls Histogramme erwnscht*/

%IF &graphics=1 %THEN %DO;

/*Globale Optionen fr Plots:*/

/*device=	     : Dateiendung fr Bilder						*/
/*gsfname=grafout: Zielverzeichnis fr Grafiken					*/
/*gsfmode=replace: Falls Datei bereits existiert, berschreiben */

GOPTIONS DEVICE=&ext   GSFNAME=grafout GSFMODE=replace FONTRES=PRESENTATION KEYMAP=NONE 
         FTEXT="Arial" HTITLE=1 HTEXT=1 XMAX=6 in YMAX=6 in XPIXELS=1800 YPIXELS=1800;

FILENAME grafout &path;

PROC UNIVARIATE DATA=&data NOPRINT;
	VAR &var;
	HISTOGRAM &var;
	OUTPUT OUT=&var N=N MEAN=MEAN STD=STD MIN=MIN pctlpre=P_ pctlpts=20 to 80 by 20 
					MAX=MAX MEDIAN=MEDIAN;
RUN;

%END;

/*Klassierung der stetigen Variablen*/

DATA &var;
	SET &var;

	%LET min=;
	%LET max=;
	%LET p_20=;
	%LET p_40=;
	%LET p_60=;
	%LET p_80=;

	CALL SYMPUT ("min", MIN);
	CALL SYMPUT ("max", MAX);
	CALL SYMPUT ("p_20", P_20);
	CALL SYMPUT ("P_40", P_40);
	CALL SYMPUT ("p_60", P_60);
	CALL SYMPUT ("p_80", P_80);
	
RUN;

DATA &var;
	SET &data;
	%LET c=_c;

	IF &var >=0     AND &var <&p_20 THEN &var&c=1;
	IF &var >=&p_20 AND &var <&p_40 THEN &var&c=2;
   	IF &var >=&p_40 AND &var <&p_60 THEN &var&c=3;
   	IF &var >=&p_60 AND &var <&p_80 THEN &var&c=4;
   	IF &var >=&p_80 				THEN &var&c=5;
	
RUN;

/*Formatierungen*/

PROC FORMAT;
	VALUE x 
	1='0-20%'
	2='20-40%'
	   3='40-60%'
	   4='60-80%'
	   5='80-100%';
RUN;

DATA &var;
	SET &var;
	FORMAT &var&c x.;
RUN;
	
PROC FREQ DATA=&var ;
	TITLE;
	TABLES &var&c / NOROW NOPERCENT;
RUN;

/*temporr angelegte Zwischendateien lschen*/

PROC DATASETS NOLIST;
	DELETE &var;
	RUN;
QUIT;

%LET var=;

%END;

%END;

%IF &missx=0 %THEN %DO;

PROC DATASETS NOLIST;
	DELETE nomiss;
RUN;
QUIT;

%END;

/*Sprungmarke fr Fehlerfall*/

%ende:

%MEND MISSDESCRIPTION;
